// 
// このコードは、https://demo.mescius.jp/diodocs/pdf のサンプルの一部として提供されています。
// © MESCIUS inc. All rights reserved.
// 
using System;
using System.IO;
using System.Drawing;
using GrapeCity.Documents.Pdf;
using GrapeCity.Documents.Pdf.Security;
using GrapeCity.Documents.Text;

namespace DsPdfWeb.Demos
{
    // このサンプルでは、パスワードで保護されたPDFを読み込み、そのメタデータを取得する方法を紹介しています。
    // なお、PDFのメタデータも暗号化されている場合があり、その場合はパスワードを指定せずに
    // メタデータを読み込むことができないことに留意してください。
    // 参考までに、読み込むPDFはオーナーパスワード「owner」とユーザーパスワード「user」で保護されています。
    public class NoPassGetMetadata
    {
        public int CreatePDF(Stream stream)
        {
            using var fs = File.OpenRead(Path.Combine("Resources", "PDFs", "Wetlands-password-user.pdf"));
            // パスワードで保護されたPDFをパスワードを指定せずに読み込めるようDecryptionOptionsを設定します。
            var dopt = new DecryptionOptions() { ThrowExceptionIfInvalidPassword = false };
            var docSrc = new GcPdfDocument();
            docSrc.Load(fs, dopt);

            // 結果を出力するPDFです。
            var doc = new GcPdfDocument();
            var page = doc.NewPage();
            // 結果のテキストのためのTextLayoutを設定します。
            var tl = page.Graphics.CreateTextLayout();
            tl.DefaultFormat.Font = Common.Util.getFont();
            tl.DefaultFormat.FontSize = 14;
            tl.MaxWidth = doc.PageSize.Width;
            tl.MaxHeight = doc.PageSize.Height;
            tl.MarginAll = tl.Resolution;
            var captionFmt = new TextFormat(tl.DefaultFormat) { FontBold = true, FontSize = tl.DefaultFormat.FontSize + 2 };

            // StandardSecurityHandlerRev4より前のすべてのセキュリティハンドラは常にメタデータを暗号化していましたが、
            // StandardSecurityHandlerRev4にはEncryptMetadataプロパティがあり、これを確認する必要があります。
            bool metadataIsEncrypted = true;
            if (docSrc.Security.EncryptHandler is StandardSecurityHandlerRev4 ssh)
                metadataIsEncrypted = ssh.EncryptMetadata;

            if (!metadataIsEncrypted)
            {
                Metadata m = docSrc.Metadata;
                tl.AppendLine("PDFには暗号化されていないメタデータがあります：", captionFmt);
                tl.AppendLine($"  Copyright：{m.Copyright}");
                tl.AppendLine($"  CreatorTool：{m.CreatorTool}");
                tl.AppendLine($"  CreateDate：{m.CreateDate}");
                tl.AppendLine($"  Description：{m.Description}");
                //tl.AppendLine($"  Keywords：{m.Keywords}");
                tl.AppendLine($"  ModifyDate：{m.ModifyDate}");
                tl.AppendLine($"  Producer：{m.Producer}");
                tl.AppendLine($"  Title：{m.Title}");
            }
            else
            {
                tl.AppendLine("PDFのメタデータが暗号化されているため、メタデータを取得することができません。", captionFmt);
            }

            // 結果のテキストを描画します。
            tl.PerformLayout(true);
            while (true)
            {
                var splitResult = tl.Split(null, out TextLayout rest);
                page.Graphics.DrawTextLayout(tl, PointF.Empty);
                if (splitResult != SplitResult.Split)
                    break;
                tl = rest;
                tl.MarginTop = tl.Resolution;
                page = doc.Pages.Add();
            }

            // PDF ドキュメントを保存します。
            doc.Save(stream);
            return doc.Pages.Count;
        }
    }
}
